'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.MaybeNothing = exports.MaybeJust = undefined;
exports.Just = Just;
exports.of = of;
exports.Nothing = Nothing;
exports.empty = empty;
exports.fromNullable = fromNullable;

require('./index.h');

/*::
type MaybeSign = 'just' | 'nothing'
*/

class MaybeBase {}

class MaybeJust extends MaybeBase {
  constructor(value) {
    super();
    this.value = value;
    /*:: return this */
  }

  map(fn) {
    return new MaybeJust(fn(this.value));
  }

  chain(fn) {
    return fn(this.value);
  }

  tap(fn) {
    fn(this.value);
    return this;
  }

  fold(l, r) {
    return r(this.value);
  }

  orElse() {
    return this.value;
  }

  match(transformer) {
    var { Just } = transformer;
    return Just(this.value);
  }

  promise() {
    return Promise.resolve(this.value);
  }

  equals(value) {
    if (value instanceof MaybeJust) {
      return this.value === value.value;
    }
    return this.value === value;
  }

  alt() {
    return (/*:: addS( */this
    ); /*:: ) */
  }
  both(maybe) {
    if (maybe instanceof MaybeJust) {
      return new MaybeJust([this.value, maybe.value]);
    }

    return new MaybeNothing();
  }
  and(maybe) {
    return maybe;
  }

  isJust() {
    return true;
  }
  isNothing() {
    return false;
  }
}

exports.MaybeJust = MaybeJust;
class MaybeNothing extends MaybeBase {
  constructor() {
    super();
    /*:: return this */
  }

  map() {
    return (/*:: changeT( */this
    ); /*:: ) */
  }

  chain() {
    return (/*:: changeT( */this
    ); /*:: ) */
  }

  tap() {
    return this;
  }

  fold(l /*:: , r: (x: T) => O */) {
    return l();
  }

  orElse(x) {
    return x;
  }

  match(transformer) {
    var { Nothing } = transformer;
    return Nothing();
  }

  promise() {
    return Promise.reject();
  }

  equals(value) {
    return value instanceof MaybeNothing;
  }

  alt(maybe) {
    return (/*:: addS( */maybe
    ); /*:: ) */
  }
  both() {
    return new MaybeNothing();
  }
  and() {
    return new MaybeNothing();
  }

  isJust() {
    return false;
  }
  isNothing() {
    return true;
  }
}

exports.MaybeNothing = MaybeNothing;


Object.defineProperty(MaybeJust.prototype, 'isomer', {
  value: 'just',
  enumerable: true,
  writable: false
});

Object.defineProperty(MaybeNothing.prototype, 'isomer', {
  value: 'nothing',
  enumerable: true,
  writable: false
});

function Just(value) {
  return new MaybeJust(value);
}

function of(value) {
  return new MaybeJust(value);
}

function Nothing() {
  return new MaybeNothing();
}

function empty() {
  return new MaybeNothing();
}

function fromNullable(value) {
  if (value == null) return new MaybeNothing();
  return new MaybeJust(value);
}
//# sourceMappingURL=maybe.js.map